#ifndef FEMPOST_H
#define FEMPOST_H

#include "fem.h"
#include <map>
#include <list>
#include <Eigen/Dense>

namespace fem
{

///

class FEM_API Post
{
public:
#ifndef SWIG
    bool verbose;
    Solver &solver;
    std::map<std::string, int> views;
#endif

public:
    Post(Solver &_solver);
    ~Post();

    void build_views();
    void view();
    void show_views(std::vector<std::string> const &names);
    void set_camera_3D();
    void deform(double factor=1.0);
    void write();
    
    std::vector<double> probe(std::string const &name, std::string const &grpname);    
    std::vector<double> probe(std::string const &name, size_t nodetag);

#ifndef SWIG
    friend FEM_API std::ostream &operator<<(std::ostream &out, Post const &obj);
#endif

private:
    void add_dofs_view(Problem &pbl, DOFs &dofs,
                       std::vector<Eigen::VectorXd *> &X,
                       Eigen::ArrayXXi &dofPart,
                       Eigen::ArrayXXi &dofIdx);
    void add_nodalvector_view(Problem &pbl,
                              std::string const &vname,
                              DOFs &dofs,
                              std::vector<Eigen::VectorXd *> &X,
                              Eigen::ArrayXXi &dofPart,
                              Eigen::ArrayXXi &dofIdx);
    void add_tensor_views(std::string const &vname,
                          std::vector<size_t> const &etags_all,
                          std::vector<std::vector<double>> const &tensor_all);
    void add_smooth_views(std::string const &vname,
                          std::map<size_t, std::vector<Eigen::VectorXd>> &voigt_by_node);

    int add_view(std::string const &name);
    void configure(int sview);
};

} // namespace fem

#endif // FEMPOST_H
